@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2012, 2013  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2009, 2012-2016  Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -29,9 +29,12 @@
 #include <isc/file.h>
 #include <isc/keyboard.h>
 #include <isc/mem.h>
+#include <isc/print.h>
 #include <isc/result.h>
 #include <isc/string.h>
 
+#include <pk11/site.h>
+
 #include <dns/keyvalues.h>
 #include <dns/name.h>
 
@@ -47,8 +50,10 @@
 const char *
 alg_totext(dns_secalg_t alg) {
 	switch (alg) {
+#ifndef PK11_MD5_DISABLE
 	    case DST_ALG_HMACMD5:
 		return "hmac-md5";
+#endif
 	    case DST_ALG_HMACSHA1:
 		return "hmac-sha1";
 	    case DST_ALG_HMACSHA224:
@@ -69,17 +74,23 @@ alg_totext(dns_secalg_t alg) {
  */
 dns_secalg_t
 alg_fromtext(const char *name) {
-	if (strcmp(name, "hmac-md5") == 0)
+	const char *p = name;
+	if (strncasecmp(p, "hmac-", 5) == 0)
+		p = &name[5];
+
+#ifndef PK11_MD5_DISABLE
+	if (strcasecmp(p, "md5") == 0)
 		return DST_ALG_HMACMD5;
-	if (strcmp(name, "hmac-sha1") == 0)
+#endif
+	if (strcasecmp(p, "sha1") == 0)
 		return DST_ALG_HMACSHA1;
-	if (strcmp(name, "hmac-sha224") == 0)
+	if (strcasecmp(p, "sha224") == 0)
 		return DST_ALG_HMACSHA224;
-	if (strcmp(name, "hmac-sha256") == 0)
+	if (strcasecmp(p, "sha256") == 0)
 		return DST_ALG_HMACSHA256;
-	if (strcmp(name, "hmac-sha384") == 0)
+	if (strcasecmp(p, "sha384") == 0)
 		return DST_ALG_HMACSHA384;
-	if (strcmp(name, "hmac-sha512") == 0)
+	if (strcasecmp(p, "sha512") == 0)
 		return DST_ALG_HMACSHA512;
 	return DST_ALG_UNKNOWN;
 }
@@ -125,7 +136,9 @@ generate_key(isc_mem_t *mctx, const char *randomfile, dns_secalg_t alg,
 	dst_key_t *key = NULL;
 
 	switch (alg) {
+#ifndef PK11_MD5_DISABLE
 	    case DST_ALG_HMACMD5:
+#endif
 	    case DST_ALG_HMACSHA1:
 	    case DST_ALG_HMACSHA224:
 	    case DST_ALG_HMACSHA256:
