@@ -1,9 +1,17 @@
 /*
  * Copyright (C) 2014-2017  Internet Systems Consortium, Inc. ("ISC")
  *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * Permission to use, copy, modify, and/or distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
 #include <config.h>
@@ -90,7 +98,7 @@ isc_gost_invalidate(isc_gost_t *ctx) {
 	if (ctx->handle == NULL)
 		return;
 	(void) pkcs_C_DigestFinal(ctx->session, garbage, &len);
-	isc_safe_memwipe(garbage, sizeof(garbage));
+	memset(garbage, 0, sizeof(garbage));
 	pk11_return_session(ctx);
 }
 
@@ -209,8 +217,8 @@ pkcs11gost_createctx_sign(dst_key_t *key, dst_context_t *dctx) {
 
 	for (i = 6; i <= 6; i++)
 		if (keyTemplate[i].pValue != NULL) {
-			isc_safe_memwipe(keyTemplate[i].pValue,
-					 keyTemplate[i].ulValueLen);
+			memset(keyTemplate[i].pValue, 0,
+			       keyTemplate[i].ulValueLen);
 			isc_mem_put(dctx->mctx,
 				    keyTemplate[i].pValue,
 				    keyTemplate[i].ulValueLen);
@@ -223,14 +231,14 @@ pkcs11gost_createctx_sign(dst_key_t *key, dst_context_t *dctx) {
 		(void) pkcs_C_DestroyObject(pk11_ctx->session, pk11_ctx->object);
 	for (i = 6; i <= 6; i++)
 		if (keyTemplate[i].pValue != NULL) {
-			isc_safe_memwipe(keyTemplate[i].pValue,
-					 keyTemplate[i].ulValueLen);
+			memset(keyTemplate[i].pValue, 0,
+			       keyTemplate[i].ulValueLen);
 			isc_mem_put(dctx->mctx,
 				    keyTemplate[i].pValue,
 				    keyTemplate[i].ulValueLen);
 		}
 	pk11_return_session(pk11_ctx);
-	isc_safe_memwipe(pk11_ctx, sizeof(*pk11_ctx));
+	memset(pk11_ctx, 0, sizeof(*pk11_ctx));
 	isc_mem_put(dctx->mctx, pk11_ctx, sizeof(*pk11_ctx));
 
 	return (ret);
@@ -314,8 +322,8 @@ pkcs11gost_createctx_verify(dst_key_t *key, dst_context_t *dctx) {
 
 	for (i = 5; i <= 5; i++)
 		if (keyTemplate[i].pValue != NULL) {
-			isc_safe_memwipe(keyTemplate[i].pValue,
-					 keyTemplate[i].ulValueLen);
+			memset(keyTemplate[i].pValue, 0,
+			       keyTemplate[i].ulValueLen);
 			isc_mem_put(dctx->mctx,
 				    keyTemplate[i].pValue,
 				    keyTemplate[i].ulValueLen);
@@ -328,14 +336,14 @@ pkcs11gost_createctx_verify(dst_key_t *key, dst_context_t *dctx) {
 		(void) pkcs_C_DestroyObject(pk11_ctx->session, pk11_ctx->object);
 	for (i = 5; i <= 5; i++)
 		if (keyTemplate[i].pValue != NULL) {
-			isc_safe_memwipe(keyTemplate[i].pValue,
-					 keyTemplate[i].ulValueLen);
+			memset(keyTemplate[i].pValue, 0,
+			       keyTemplate[i].ulValueLen);
 			isc_mem_put(dctx->mctx,
 				    keyTemplate[i].pValue,
 				    keyTemplate[i].ulValueLen);
 		}
 	pk11_return_session(pk11_ctx);
-	isc_safe_memwipe(pk11_ctx, sizeof(*pk11_ctx));
+	memset(pk11_ctx, 0, sizeof(*pk11_ctx));
 	isc_mem_put(dctx->mctx, pk11_ctx, sizeof(*pk11_ctx));
 
 	return (ret);
@@ -359,7 +367,7 @@ pkcs11gost_destroyctx(dst_context_t *dctx) {
 			(void) pkcs_C_DestroyObject(pk11_ctx->session,
 					       pk11_ctx->object);
 		pk11_return_session(pk11_ctx);
-		isc_safe_memwipe(pk11_ctx, sizeof(*pk11_ctx));
+		memset(pk11_ctx, 0, sizeof(*pk11_ctx));
 		isc_mem_put(dctx->mctx, pk11_ctx, sizeof(*pk11_ctx));
 		dctx->ctxdata.pk11_ctx = NULL;
 	}
@@ -566,7 +574,7 @@ pkcs11gost_generate(dst_key_t *key, int unused, void (*callback)(int)) {
 	(void) pkcs_C_DestroyObject(pk11_ctx->session, priv);
 	(void) pkcs_C_DestroyObject(pk11_ctx->session, pub);
 	pk11_return_session(pk11_ctx);
-	isc_safe_memwipe(pk11_ctx, sizeof(*pk11_ctx));
+	memset(pk11_ctx, 0, sizeof(*pk11_ctx));
 	isc_mem_put(key->mctx, pk11_ctx, sizeof(*pk11_ctx));
 
 	return (ISC_R_SUCCESS);
@@ -578,7 +586,7 @@ pkcs11gost_generate(dst_key_t *key, int unused, void (*callback)(int)) {
 	if (pub != CK_INVALID_HANDLE)
 		(void) pkcs_C_DestroyObject(pk11_ctx->session, pub);
 	pk11_return_session(pk11_ctx);
-	isc_safe_memwipe(pk11_ctx, sizeof(*pk11_ctx));
+	memset(pk11_ctx, 0, sizeof(*pk11_ctx));
 	isc_mem_put(key->mctx, pk11_ctx, sizeof(*pk11_ctx));
 
 	return (ret);
@@ -612,8 +620,7 @@ pkcs11gost_destroy(dst_key_t *key) {
 		case CKA_VALUE:
 		case CKA_VALUE2:
 			if (attr->pValue != NULL) {
-				isc_safe_memwipe(attr->pValue,
-						 attr->ulValueLen);
+				memset(attr->pValue, 0, attr->ulValueLen);
 				isc_mem_put(key->mctx,
 					    attr->pValue,
 					    attr->ulValueLen);
@@ -621,11 +628,12 @@ pkcs11gost_destroy(dst_key_t *key) {
 			break;
 		}
 	if (gost->repr != NULL) {
-		isc_safe_memwipe(gost->repr, gost->attrcnt * sizeof(*attr));
+		memset(gost->repr, 0, gost->attrcnt * sizeof(*attr));
 		isc_mem_put(key->mctx,
-			    gost->repr, gost->attrcnt * sizeof(*attr));
+			    gost->repr,
+			    gost->attrcnt * sizeof(*attr));
 	}
-	isc_safe_memwipe(gost, sizeof(*gost));
+	memset(gost, 0, sizeof(*gost));
 	isc_mem_put(key->mctx, gost, sizeof(*gost));
 	key->keydata.pkey = NULL;
 }
@@ -693,8 +701,7 @@ pkcs11gost_fromdns(dst_key_t *key, isc_buffer_t *data) {
 		switch (attr->type) {
 		case CKA_VALUE:
 			if (attr->pValue != NULL) {
-				isc_safe_memwipe(attr->pValue,
-						 attr->ulValueLen);
+				memset(attr->pValue, 0, attr->ulValueLen);
 				isc_mem_put(key->mctx,
 					    attr->pValue,
 					    attr->ulValueLen);
@@ -702,11 +709,12 @@ pkcs11gost_fromdns(dst_key_t *key, isc_buffer_t *data) {
 			break;
 		}
 	if (gost->repr != NULL) {
-		isc_safe_memwipe(gost->repr, gost->attrcnt * sizeof(*attr));
+		memset(gost->repr, 0, gost->attrcnt * sizeof(*attr));
 		isc_mem_put(key->mctx,
-			    gost->repr, gost->attrcnt * sizeof(*attr));
+			    gost->repr,
+			    gost->attrcnt * sizeof(*attr));
 	}
-	isc_safe_memwipe(gost, sizeof(*gost));
+	memset(gost, 0, sizeof(*gost));
 	isc_mem_put(key->mctx, gost, sizeof(*gost));
 	return (ISC_R_NOMEMORY);
 }
@@ -765,7 +773,7 @@ pkcs11gost_tofile(const dst_key_t *key, const char *directory) {
 	ret = dst__privstruct_writefile(key, &priv, directory);
 
 	if (buf != NULL) {
-		isc_safe_memwipe(buf, attr->ulValueLen);
+		memset(buf, 0, attr->ulValueLen);
 		isc_mem_put(key->mctx, buf, attr->ulValueLen);
 	}
 	return (ret);
@@ -808,7 +816,7 @@ pkcs11gost_tofile(const dst_key_t *key, const char *directory) {
 	ret = dst__privstruct_writefile(key, &priv, directory);
 
 	if (buf != NULL) {
-		isc_safe_memwipe(buf, attr->ulValueLen);
+		memset(buf, 0, attr->ulValueLen);
 		isc_mem_put(key->mctx, buf, attr->ulValueLen);
 	}
 	return (ret);
@@ -840,7 +848,7 @@ pkcs11gost_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 		key->key_size = pub->key_size;
 
 		dst__privstruct_free(&priv, mctx);
-		isc_safe_memwipe(&priv, sizeof(priv));
+		memset(&priv, 0, sizeof(priv));
 
 		return (ISC_R_SUCCESS);
 	}
@@ -899,14 +907,14 @@ pkcs11gost_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	attr->ulValueLen = priv.elements[0].length;
 
 	dst__privstruct_free(&priv, mctx);
-	isc_safe_memwipe(&priv, sizeof(priv));
+	memset(&priv, 0, sizeof(priv));
 
 	return (ISC_R_SUCCESS);
 
  err:
 	pkcs11gost_destroy(key);
 	dst__privstruct_free(&priv, mctx);
-	isc_safe_memwipe(&priv, sizeof(priv));
+	memset(&priv, 0, sizeof(priv));
 	return (ret);
 }
 
